@@ -1,33 +1,31 @@
 package OpenGL::Sandbox;
-BEGIN { $OpenGL::Sandbox::VERSION = '0.03'; }
+BEGIN { $OpenGL::Sandbox::VERSION = '0.04'; }
 use v5.14; # I can aim for older upon request.  Not expecting any requests though.
-use strict;
-use warnings;
+use Exporter::Extensible -exporter_setup => 1;
 use Try::Tiny;
-use Exporter;
 use Carp;
 use Log::Any '$log';
+use Module::Runtime 'require_module';
+use Scalar::Util 'weaken';
 # Choose OpenGL::Modern if available, else fall back to OpenGL.
 # But use the one configured in the environment.  But yet don't blindly
 # load modules from environment either.
 our $OpenGLModule;
 BEGIN {
-	$OpenGLModule //= do {
-		my $fromenv= $ENV{OPENGL_SANDBOX_OPENGLMODULE} // '';
-		# Don't blindly require module from environment...
-		# Any other value, and the user must require it themself (such as perl -M)
-		eval "require $fromenv" if $fromenv eq 'OpenGL' || $fromenv eq 'OpenGL::Modern';
-		$fromenv? $fromenv
+	my $fromenv= $ENV{OPENGL_SANDBOX_OPENGLMODULE} // '';
+	# Don't blindly require module from environment...
+	# Any other value, and the user must require it themself (such as perl -M)
+	require_module($fromenv) if $fromenv eq 'OpenGL' || $fromenv eq 'OpenGL::Modern';
+	my $mod= $fromenv? $fromenv
 		: eval 'require OpenGL::Modern; 1'? 'OpenGL::Modern'
 		: eval 'require OpenGL; 1'? 'OpenGL'
 		: croak "Can't load either OpenGL::Modern or OpenGL.  Please install one.";
-	};
-	
 	# If this succeeds, assume it is safe to eval this package name later
-	$OpenGLModule->import(qw/
-		glGetString glGetError
-		GL_VERSION
+	$mod->import(qw/
+		glGetString glGetError glClear
+		GL_VERSION GL_COLOR_BUFFER_BIT GL_DEPTH_BUFFER_BIT
 	/);
+	$OpenGLModule= $mod;
 }
 require constant;
 require OpenGL::Sandbox::ResMan;
@@ -39,73 +37,51 @@ sub tex  { OpenGL::Sandbox::ResMan->default_instance->tex(@_) }
 sub font { OpenGL::Sandbox::ResMan->default_instance->font(@_) }
 
 
-our @EXPORT_OK= qw( font tex make_context get_gl_errors );
-our %EXPORT_TAGS= ( all => \@EXPORT_OK );
+export qw( =$res font tex make_context current_context next_frame
+	get_gl_errors log_gl_errors warn_gl_errors
+	glGetString glGetError GL_VERSION ),
+	-V1 => sub { Module::Runtime::use_module('OpenGL::Sandbox::V1','0.04'); };
 
-sub import {
-	my $caller= caller;
-	my $class= $_[0];
-	my @gl_const;
-	my @gl_fn;
-	for (reverse 1..$#_) {
-		my $arg= $_[$_];
-		if ($arg eq '$res') {
-			my $res= OpenGL::Sandbox::ResMan->default_instance;
-			no strict 'refs';
-			*{$caller.'::res'}= \$res;
-			splice(@_, $_, 1);
-		}
-		elsif ($arg =~ /^:V(\d)(:.*)?$/) {
-			my $mod= "OpenGL::Sandbox::V$1";
-			my $imports= $2;
-			$imports =~ s/:/ :/g;
-			eval "package $caller; use $mod qw/ $imports /; 1"
-				or croak "Can't load $mod (note that this must be installed separately)\n  $@";
-			splice(@_, $_, 1);
-		}
-		elsif ($arg =~ /^GL_/) {
-			push @gl_const, $arg;
-			splice(@_, $_, 1);
-		}
-		elsif ($arg =~ /^gl[a-zA-Z]/) {
-			# Let local methods in this package override external ones
-			unless ($class->can($arg)) {
-				push @gl_fn, $arg;
-				splice(@_, $_, 1);
-			}
+sub _generateScalar_res { \OpenGL::Sandbox::ResMan->default_instance; }
+
+sub exporter_autoload_symbol {
+	my ($self, $sym)= @_;
+	if ($sym =~ /^(?:(GL_)|(gl[a-zA-Z]))/) {
+		# First import it into this package, for cached export
+		$OpenGLModule->import($sym);
+		no strict 'refs';
+		# If it is a constant, make sure it has been collapsed to a perl constant
+		# (the original OpenGL module fails to do this)
+		if ($1) {
+			my $val= __PACKAGE__->can($sym)->();
+			undef *$sym;
+			constant->import($sym => $val);
 		}
+		return ($OpenGL::Sandbox::EXPORT{$sym}= __PACKAGE__->can($sym));
 	}
-	$class->_import_gl_constants_into($caller, @gl_const) if @gl_const;
-	$class->_import_gl_functions_into($caller, @gl_fn) if @gl_fn;
-	# Let the real Exporter module handle anything remaining in @_
-	goto \&Exporter::import;
-}
-
-sub _import_gl_constants_into {
-	my ($class, $into, @names)= @_;
-	# First, import into this module, then import into caller.  This resolves an
-	# inefficiency in traditional OpenGL module where it optimizes imports for
-	# import speed rather than runtime speed.  We want constants to actually be
-	# perl constants.
-	my @need_import_const= grep !$class->can($_), @names;
-	$OpenGLModule->import(@need_import_const);
-	# Now for each constant we imported, undefine it then pass it to the constant module
-	no strict 'refs';
-	for (@need_import_const) {
-		my $val= $class->can($_)->();
-		undef *$_;
-		constant->import($_ => $val);
+	# Notation of -V1 means require "OpenGL::Sandbox::V1".
+	elsif ($sym =~ /^-V([0-9]+)/) {
+		my $mod= "OpenGL::Sandbox::V$1";
+		return [ sub { require_module($mod) }, 0 ];
 	}
-	# Now import them all into caller
-	*{ $into . '::' . $_ }= $class->can($_) for @names;
-}
-
-sub _import_gl_functions_into {
-	my ($class, $into, @names)= @_;
-	eval "package $into; $OpenGLModule->import(\@names); 1" or die $@;
+	return $self->next::method($sym);
 }
 
 
+our %context_provider_aliases;
+BEGIN {
+	%context_provider_aliases= (
+		'GLX'            => 'OpenGL::Sandbox::ContextShim::GLX',
+		'X11::GLX'       => 'OpenGL::Sandbox::ContextShim::GLX',
+		'X11::GLX::DWIM' => 'OpenGL::Sandbox::ContextShim::GLX',
+		'GLFW'           => 'OpenGL::Sandbox::ContextShim::GLFW',
+		'OpenGL::GLFW'   => 'OpenGL::Sandbox::ContextShim::GLFW',
+		'SDL'            => 'OpenGL::Sandbox::ContextShim::SDL',
+		'SDLx::App'      => 'OpenGL::Sandbox::ContextShim::SDL',
+	);
+}
+
+our $current_context;
 sub make_context {
 	my (%opts)= @_;
 	# Check for geometry specification on command line
@@ -126,78 +102,37 @@ sub make_context {
 		$opts{x} //= $l if defined $l;
 		$opts{y} //= $t if defined $t;
 	}
-	# Try X11 first, because lightest weight
+	
+	# Load user's requested provider, or auto-detect first available
 	my $provider= $ENV{OPENGL_SANDBOX_CONTEXT_PROVIDER};
 	$provider //=
-		eval('require X11::GLX::DWIM; 1;') ? 'X11::GLX::DWIM'
-		: eval('require SDLx::App; 1;') ? 'SDLx::App'
-		: croak "make_context needs one of X11::GLX or SDL to be installed";
-	if ($provider eq 'X11::GLX' || $provider eq 'X11::GLX::DWIM') {
-		require X11::GLX::DWIM;
-		my $glx= X11::GLX::DWIM->new();
-		my $visible= $opts{visible} // 1;
-		if ($visible) {
-			$glx->target({ window => {
-				x => $opts{x} // 0,
-				y => $opts{y} // 0,
-				width => $opts{width} // 400,
-				height => $opts{height} // 400
-			}});
-		} else {
-			$glx->target({ pixmap => {
-				width => $opts{width} // 256,
-				height => $opts{height} // 256
-			}});
-		}
-		$log->infof("Loaded X11::GLX::DWIM %s, target '%s', GLX Version %s, OpenGL version %s\n",
-			$glx->VERSION, $visible? 'window':'pixmap', $glx->glx_version, glGetString(GL_VERSION));
-		return $glx;
-	}
-	# TODO: Else try GLFW
-	# Else try SDL
-	elsif ($provider eq 'SDL' || $provider eq 'SDLx::App') {
-		my $sdl_subclass= _init_sdl_wrapper();
-		# TODO: Figure out best way to create invisible SDL window
-		if (defined $opts{visible} && !$opts{visible}) {
-			$opts{x}= -100;
-			$opts{width}= $opts{height}= 1;
-		}
-		# This is the only option I know of for SDL to set initial window placement
-		local $ENV{SDL_VIDEO_WINDOW_POS}= ($opts{x}//0).','.($opts{y}//0)
-			if defined $opts{x} || defined $opts{y};
-		my $flags= 0;
-		$flags |= SDL::SDL_NOFRAME() if $opts{noframe};
-		$flags |= SDL::SDL_FULLSCREEN() if $opts{fullscreen};
-		my $sdl= $sdl_subclass->new(
-			title  => $opts{title} // 'OpenGL',
-			(defined $opts{width}?  ( width  => $opts{width} ) : ()),
-			(defined $opts{height}? ( height => $opts{height} ) : ()),
-			($flags?                ( flags => (SDL::SDL_ANYFORMAT() | $flags) ) : ()),
-			opengl => 1,
-			exit_on_quit => 1,
-		);
-		$log->infof("Loaded SDLx::App %s, OpenGL version %s\n", $sdl->VERSION, glGetString(GL_VERSION));
-		return $sdl;
-	}
-	# TODO: else try Prima
-	else {
-		die "Unhandled context provider $provider";
-	}
+		# Try X11 first, because lightest weight
+		eval('require X11::GLX::DWIM; 1;') ? 'GLX'
+		: eval('require OpenGL::GLFW; 1;') ? 'GLFW'
+		: eval('require SDLx::App; 1;') ? 'SDL'
+		: croak "make_context needs one of X11::GLX, OpenGL::GLFW, or SDLx::App to be installed";
+	
+	my $class= $context_provider_aliases{$provider}
+		or croak "Unhandled context provider $provider";
+	require_module($class);
+	
+	undef $current_context;
+	my $cx= $current_context= $class->new(%opts);
+	$log->infof("Loaded %s", $cx->context_info);
+	weaken($current_context) if defined wantarray;
+	return $cx;
 }
 
-sub _init_sdl_wrapper {
-	# Hack together a subclass of SDLx::App, but without alerting CPAN to its presence
-	# and without introducing it to the perl namespace unless SDLx::App exists.
-	unless (OpenGL::Sandbox::SDLx::App->VERSION) {
-		require SDLx::App;
-		no warnings 'once';
-		$OpenGL::Sandbox::SDLx::App::VERSION= __PACKAGE__->VERSION;
-		push @OpenGL::Sandbox::SDLx::App::ISA, 'SDLx::App';
-		*OpenGL::Sandbox::SDLx::App::swap_buffers= sub {
-			shift->sync;
-		};
-	}
-	return 'OpenGL::Sandbox::SDLx::App';
+
+sub current_context { $current_context }
+
+
+sub next_frame() {
+	my $gl= OpenGL::Sandbox::current_context();
+	$gl->swap_buffers if $gl;
+	warn_gl_errors();
+	glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
+	__PACKAGE__->maybe::next::method();
 }
 
 
@@ -223,6 +158,16 @@ sub get_gl_errors {
 	return @names;
 }
 
+sub log_gl_errors {
+	my @errors= get_gl_errors;
+	$log->error("GL Error Bits: ".join(', ', @errors)) if @errors;
+}
+
+sub warn_gl_errors {
+	my @errors= get_gl_errors;
+	warn("GL Error Bits: ".join(', ', @errors)."\n") if @errors;
+}
+
 1;
 
 __END__
@@ -237,7 +182,40 @@ OpenGL::Sandbox - Rapid-prototyping utilities for OpenGL
 
 =head1 VERSION
 
-version 0.03
+version 0.04
+
+=head1 SYNOPSIS
+
+  use OpenGL::Sandbox qw( -V1 :all );
+  make_context;
+  setup_projection;
+  next_frame;
+  scale .01;
+  font('Arial.ttf')->render("Hello World");
+  next_frame;
+
+=head1 DESCRIPTION
+
+This module collection aims to provide quick and easy access to OpenGL, for use in one-liners
+or experiments or simple visualizations.  There are many system dependencies involved, so to
+make the modules more accessible I divided it into multiple distributions:
+
+=over
+
+=item OpenGL::Sandbox
+
+This package, containing the foundation for the rest, and anything in common among the
+different OpenGL APIs.  It has methods to help set up the context, and load textures.
+
+=item L<OpenGL::Sandbox::V1>
+
+Everything related to OpenGL 1.x API
+
+=item L<OpenGL::Sandbox::V1::FTGLFont>
+
+A class providing fonts, but only for OpenGL 1.x
+
+=back
 
 =head1 EXPORTS
 
@@ -264,14 +242,18 @@ Shortcut for C<< OpenGL::Sandbox::ResMan->default_instance->font >>
 Note that you need to install L<OpenGL::Sandbox::V1::FTGLFont> in order to get font support,
 currently.  Other font providers might be added later.
 
-=head2 :V1:all
+=head2 -V1
 
-Exports ':all' from L<OpenGL::Sandbox::V1> (which must be installed separately).
-This module contains many "sugar" functions to make the GL 1.x API more friendly.
+Loads L<OpenGL::SandBox::V1> (which must be installed separately) and makes all its symbols
+available for importing.  This module contains many "sugar" functions to make the
+GL 1.x API more friendly.
 
-C<:V2:all>, C<:V3:all>, etc will likewise import everything from packages named
-C<OpenGL::SandBox::V$_> which do not currently exist, but could be authored
-in the future.
+C<-V2>, C<-V3>, etc will likewise import everything from packages named C<OpenGL::SandBox::V$_>
+which do not currently exist, but could be authored in the future.
+
+=head2 :all
+
+This *only* exports the symbols defined by this module collection, *not* every OpenGL symbol.
 
 =head2 make_context
 
@@ -283,8 +265,9 @@ with environment variable C<OPENGL_SANDBOX_CONTEXT_PROVIDER>.
 It assumes you don't have any desire to receive user input and just want to render some stuff.
 If you do actually have a preference, you should just invoke that package yourself.
 
-Always returns an object whose scope controls the lifecycle of the window, and that object
-always has a C<swap_buffers> method.
+Returns an object whose scope controls the lifecycle of the window, and that object always has
+a C<swap_buffers> method.  If you call this method in void context, the object is stored
+internally and you can access it with L</current_context>.
 
 This attempts to automatically pick up the window geometry, either from a "--geometry=" option
 or from the environment variable C<OPENGL_SANDBOX_GEOMETRY>.  The Geometry value is in X11
@@ -321,10 +304,45 @@ Note that if you're using Classic OpenGL (V1) you also need to set up the projec
 to something more useful than the defaults before rendering anything.
 See L<OpenGL::Sandbox::V1/setup_projection>.
 
+=head2 current_context
+
+Returns the most recently created result of L</make_context>, assuming it hasn't been
+garbage-collected.  If you stored the return value of L</make_context>, then garbage
+collection happens according to that reference.  If you called L</make_context> in void
+context, then the GL context will live indefinitely.
+
+If you have a simple program with only one context, this global simplifies life for you.
+
+=head2 next_frame
+
+This calls a sequence of:
+
+  current_context->swap_buffers;
+  warn_gl_errors;
+  glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
+
+If you have loaded "-V1" it also calls
+
+  glLoadIdentity();
+
+Note that C<< current_context->swap_buffers() >> will only work after L</make_context>.
+
+This is intended to help out with quick prototyping and one-liners:
+
+  perl -e 'use OpenGL::Sandbox qw( -V1 :all ); make_context; while(1) { next_frame; ...; }'
+
 =head2 get_gl_errors
 
 Returns the symbolic names of any pending OpenGL errors, as a list.
 
+=head2 log_gl_errors
+
+Write all GL errors to Log::Any as C<< ->error >>
+
+=head2 warn_gl_errors
+
+Emit any GL errors using 'warn'
+
 =head1 INSTALLING
 
 Getting this module collection installed is abnormally difficult.  This is a "selfish module"
@@ -358,7 +376,7 @@ L<File::Map>, for efficiently memory-mapping resource files
 
 =item *
 
-L<Inline::C>, including a local C compiler
+XS support (C compiler)
 
 =back
 
@@ -370,10 +388,6 @@ For the "V1" module (L<OpenGL::Sandbox::V1>) you will additionally need
 
 libGLU and headers
 
-=item *
-
-Inline::CPP, including a local C++ compiler
-
 =back
 
 For the "FTGLFont" module (L<OpenGL::Sandbox::V1::FTGLFont>) you will additionally need
@@ -387,8 +401,8 @@ libftgl, and libfreetype2, and headers
 =back
 
 You probably also want a module to open a GL context to see things in.  This module is aware
-of L<X11::GLX> and L<SDL>, but you can use anything you like since the GL context
-is global.
+of L<OpenGL::GLFW>, L<X11::GLX> and L<SDL>, but you can use anything you like since the GL
+context is global.
 
 =head1 AUTHOR
 
@@ -396,7 +410,7 @@ Michael Conrad <mike@nrdvana.net>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2018 by Michael Conrad.
+This software is copyright (c) 2019 by Michael Conrad.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
