@@ -1,8 +1,8 @@
 package OpenGL::Sandbox::ContextShim::GLFW;
+BEGIN { $OpenGL::Sandbox::ContextShim::GLFW::VERSION = '0.04'; }
 use strict;
 use warnings;
 use Carp;
-use Scalar::Util 'weaken';
 use OpenGL::GLFW qw/ glfwInit glfwGetVersionString glfwTerminate NULL GLFW_TRUE GLFW_FALSE
 	glfwGetPrimaryMonitor glfwCreateWindow glfwMakeContextCurrent glfwDestroyWindow
 	glfwSwapInterval glfwSwapBuffers glfwPollEvents
@@ -11,12 +11,10 @@ use OpenGL::GLFW qw/ glfwInit glfwGetVersionString glfwTerminate NULL GLFW_TRUE
 use OpenGL::Sandbox qw/ glGetString GL_VERSION /;
 
 # ABSTRACT: Context wrapper around OpenGL::GLFW API
-our $VERSION = '0.049_001'; # TRIAL VERSION
 
 # would use Moo, but I want to write my own constructor rather than store
 # all these arguments as official attributes.
 our $glfw_init;
-our %instances;
 sub new {
 	my $class= shift;
 	my %opts= ref $_[0] eq 'HASH'? %{$_[0]} : @_;
@@ -42,25 +40,15 @@ sub new {
 	
 	glfwMakeContextCurrent($w);
 	glfwSwapInterval(1) if $opts{vsync} // 1;
-	
-	weaken($instances{$self}= $self);
 	return $self;
 }
 
 sub DESTROY {
 	my $self= shift;
 	glfwDestroyWindow(delete $self->{window}) if defined $self->{window};
-	delete $instances{$self};
 }
 
-END {
-	for (values %instances) {
-		glfwDestroyWindow(delete $_->{window}) if defined $_->{window};
-	}
-	glfwTerminate if $glfw_init;
-}
-
-sub window { shift->{window} }
+END { glfwTerminate if $glfw_init }
 
 sub context_info {
 	my $self= shift;
@@ -70,10 +58,8 @@ sub context_info {
 
 sub swap_buffers {
 	my $self= shift;
-	if ($self->window) {
-		glfwSwapBuffers($self->window);
-		glfwPollEvents;
-	}
+	glfwSwapBuffers($self->{window});
+	glfwPollEvents;
 }
 
 1;
@@ -90,28 +76,19 @@ OpenGL::Sandbox::ContextShim::GLFW - Context wrapper around OpenGL::GLFW API
 
 =head1 VERSION
 
-version 0.049_001
+version 0.04
 
 =head1 DESCRIPTION
 
 This class is loaded automatically if needed by L<OpenGL::Sandbox/make_context>.
-It uses L<OpenGL::GLFW> to create an OpenGL context.
 
-=head1 ATTRIBUTES
-
-=head2 window
-
-The GLFW window handle
-
-=head1 METHODS
-
-=head2 Standard ContextShim API:
+It provides
 
 =over 14
 
 =item new
 
-Accepting all the options of L<OpenGL::Sandbox/make_context>
+Accepting all the options of make_context
 
 =item context_info
 
