@@ -1,20 +1,15 @@
 package OpenGL::Sandbox::ContextShim::GLX;
-use strict;
-use warnings;
-use Carp;
-use Scalar::Util 'weaken';
-use OpenGL::Sandbox qw/ glGetString GL_VERSION /;
-use X11::GLX::DWIM;
+BEGIN { $OpenGL::Sandbox::ContextShim::GLX::VERSION = '0.04'; }
+use parent 'X11::GLX::DWIM';
+use OpenGL::Sandbox 'glGetString', 'GL_VERSION';
 
 # ABSTRACT: Subclass of X11::GLX::DWIM to meet contract of OpenGL::Sandbox::make_context
-our $VERSION = '0.049_001'; # TRIAL VERSION
 
-our %instances;
 sub new {
 	my $class= shift;
 	my %opts= ref $_[0] eq 'HASH'? %{$_[0]} : @_;
 	my $visible= $opts{visible} // 1;
-	my $glx= X11::GLX::DWIM->new();
+	my $glx= $class->SUPER::new();
 	# Target is lazy.  Make sure GL context fully initialized before return.
 	if ($visible) {
 		$glx->target({ window => {
@@ -29,27 +24,13 @@ sub new {
 			height => $opts{height} // 256
 		}});
 	}
-	my $self= bless { glx => $glx }, $class;
-	weaken($instances{$self}= $self);
-	return $self;
+	return $glx;
 }
 
-END {
-	delete $_->{glx} for values %instances;
-}
-
-sub glx { shift->{glx} }
-
 sub context_info {
 	my $self= shift;
 	sprintf("X11::GLX::DWIM %s, target '%s', GLX Version %s, OpenGL version %s\n",
-		$self->glx && $self->glx->VERSION, $self->glx && $self->glx->target,
-		$self->glx && $self->glx->glx_version, glGetString(GL_VERSION));
-}
-
-sub swap_buffers {
-	my $self= shift;
-	$self->glx->swap_buffers if $self->glx;
+		$self->SUPER::VERSION, $self->target, $self->glx_version, glGetString(GL_VERSION));
 }
 
 1;
@@ -66,35 +47,26 @@ OpenGL::Sandbox::ContextShim::GLX - Subclass of X11::GLX::DWIM to meet contract
 
 =head1 VERSION
 
-version 0.049_001
+version 0.04
 
 =head1 DESCRIPTION
 
 This class is loaded automatically if needed by L<OpenGL::Sandbox/make_context>.
-It uses L<X11::GLX::DWIM> to create an OpenGL context.
 
-=head1 ATTRIBUTES
-
-=head2 glx
-
-The L<X11::GLX::DWIM> object
-
-=head1 METHODS
-
-=head2 Standard ContextShim API:
+It provides
 
 =over 14
 
 =item new
 
-Accepting all the options of L<OpenGL::Sandbox/make_context>
+Accepting all the options of make_context
 
 =item context_info
 
-=item swap_buffers
-
 =back
 
+It also a subclass of X11::GLX::DWIM so you can call those methods on it too.
+
 =head1 AUTHOR
 
 Michael Conrad <mike@nrdvana.net>
