@@ -1,15 +1,10 @@
 package OpenGL::Sandbox::ContextShim::SDL;
-use strict;
-use warnings;
-use Carp;
-use Scalar::Util 'weaken';
-use OpenGL::Sandbox qw/ glGetString GL_VERSION /;
-use SDLx::App;
+BEGIN { $OpenGL::Sandbox::ContextShim::SDL::VERSION = '0.04'; }
+use parent 'SDLx::App';
+use OpenGL::Sandbox 'glGetString', 'GL_VERSION';
 
 # ABSTRACT: Subclass of SDLx::App to meet contract of OpenGL::Sandbox::make_context
-our $VERSION = '0.049_001'; # TRIAL VERSION
 
-my %instances;
 sub new {
 	my $class= shift;
 	my %opts= ref $_[0] eq 'HASH'? %{$_[0]} : @_;
@@ -24,7 +19,7 @@ sub new {
 	my $flags= 0;
 	$flags |= SDL::SDL_NOFRAME() if $opts{noframe};
 	$flags |= SDL::SDL_FULLSCREEN() if $opts{fullscreen};
-	my $sdl= SDLx::App->new(
+	$class->SUPER::new(
 		title  => $opts{title} // 'OpenGL',
 		(defined $opts{width}?  ( width  => $opts{width} ) : ()),
 		(defined $opts{height}? ( height => $opts{height} ) : ()),
@@ -32,26 +27,15 @@ sub new {
 		opengl => 1,
 		exit_on_quit => 1,
 	);
-	my $self= bless { sdl => $sdl }, $class;
-	weaken($instances{$self}= $self);
-	return $self;
 }
 
-END {
-	delete $_->{sdl} for values %instances;
-}
-
-sub sdl { shift->{sdl} }
-
 sub context_info {
 	my $self= shift;
-	sprintf("SDLx::App %s, OpenGL version %s\n",
-		$self->sdl && $self->sdl->VERSION, glGetString(GL_VERSION));
+	sprintf("SDLx::App %s, OpenGL version %s\n", $self->SUPER::VERSION, glGetString(GL_VERSION));
 }
 
 sub swap_buffers {
-	my $self= shift;
-	$self->sdl->sync if $self->sdl;
+	shift->sync;
 }
 
 1;
@@ -68,28 +52,19 @@ OpenGL::Sandbox::ContextShim::SDL - Subclass of SDLx::App to meet contract of Op
 
 =head1 VERSION
 
-version 0.049_001
+version 0.04
 
 =head1 DESCRIPTION
 
 This class is loaded automatically if needed by L<OpenGL::Sandbox/make_context>.
-It uses L<SDLx::App> to create an OpenGL context.
 
-=head1 ATTRIBUTES
-
-=head2 sdl
-
-The SDLx::App object
-
-=head1 METHODS
-
-=head2 Standard ContextShim API:
+It provides
 
 =over 14
 
 =item new
 
-Accepting all the options of L<OpenGL::Sandbox/make_context>
+Accepting all the options of make_context
 
 =item context_info
 
@@ -97,6 +72,8 @@ Accepting all the options of L<OpenGL::Sandbox/make_context>
 
 =back
 
+It also a subclass of SDLx::App so you can call those methods on it too.
+
 =head1 AUTHOR
 
 Michael Conrad <mike@nrdvana.net>
